#!/usr/bin/env ruby

module GClouder
  module Resources
    module Compute
      module BackendBuckets
        include GClouder::Logging
        include GClouder::GCloud
        include GClouder::Resource::Cleaner

        def self.header(stage = :ensure)
          info "[#{stage}] compute / backend-buckets", indent: 1, title: true
        end

        def self.ensure
          return if Local.list.empty?
          header
          Local.list.each do |region, buckets|
            info region, heading: true, indent: 2
            info
            buckets.each do |bucket|
              BackendBucket.ensure(bucket["name"], bucket)
            end
          end
        end

        def self.validate
          return if Local.list.empty?
          header :validate
          Local.validate
        end

        def self.check
        end

        module Local
          include GClouder::Config::Project
          include GClouder::Logging

          def self.validate
            info "global", heading: true, indent: 2
            Resources::Validate::Global.instances(
              list,
              required_keys:  GClouder::Config::Arguments.required(%w(compute backend-buckets)),
              permitted_keys: GClouder::Config::Arguments.permitted(%w(compute backend-buckets)),
            )
          end

          def self.list
            Resources::Global.instances(path: %w(compute backend-buckets))
          end
        end

        module Remote
          def self.list
            Resources::Remote.instances(path: %w(compute backend-buckets))
          end
        end

        module BackendBucket
          include GClouder::GCloud

          def self.ensure(name, args = {}, silent: false)
            Resource.ensure :"compute backend-buckets", name, hash_to_args(args), silent: silent
          end
        end
      end
    end
  end
end
