#!/usr/bin/env ruby

module GClouder
  module Resources
    module Compute
      module TargetHTTPSProxies
        include GClouder::Logging
        include GClouder::GCloud
        include GClouder::Resource::Cleaner

        def self.header(stage = :ensure)
          info "[#{stage}] compute / target-https-proxies", indent: 1, title: true
        end

        def self.ensure
          return if Local.list.empty?
          header
          Local.list.each do |region, rules|
            info region, heading: true, indent: 2
            info
            rules.each do |rule|
              Rule.ensure(rule["name"], rule)
            end
          end
        end

        def self.validate
          return if Local.list.empty?
          header :validate
          Local.validate
        end

        def self.check
        end

        module Local
          include GClouder::Config::Project
          include GClouder::Logging

          def self.validate
            info "global", heading: true, indent: 2
            Resources::Validate::Global.instances(
              list,
              required_keys:  GClouder::Config::Arguments.required(%w(compute target-https-proxies)),
              permitted_keys: GClouder::Config::Arguments.permitted(%w(compute target-https-proxies))
            )
          end

          def self.list
            Resources::Global.instances(path: %w(compute target_https_proxies))
          end
        end

        module Remote
          def self.list
            Resources::Remote.instances(path: %w(compute target-https-proxies))
          end
        end

        module Rule
          include GClouder::GCloud

          def self.ensure(rule, args = {}, silent: false)
            Resource.ensure :"compute target-https-proxies", rule, hash_to_args(args), silent: silent
          end
        end
      end
    end
  end
end
