#!/usr/bin/env ruby

module GClouder
  module Resources
    module Compute
      module URLMaps
        include GClouder::Logging
        include GClouder::GCloud
        include GClouder::Resource::Cleaner

        def self.header(stage = :ensure)
          info "[#{stage}] compute / url-maps", indent: 1, title: true
        end

        def self.ensure
          return if Local.list.empty?
          header
          Local.list.each do |region, url_maps|
            info region, heading: true, indent: 2
            info
            url_maps.each do |url_map|
              URLMap.ensure(url_map["name"], url_map)

              next unless url_map.key?("path_matchers")
              url_map["path_matchers"].each do |path_matcher|
                PathMatcher.ensure(url_map["name"], path_matcher)
              end
            end
          end
        end

        def self.validate
          return if Local.list.empty?
          header :validate
          Local.validate
        end

        def self.check
        end

        module Local
          include GClouder::Config::Project
          include GClouder::Logging

          def self.validate
            info "global", heading: true, indent: 2
            Resources::Validate::Global.instances(
              list,
              required_keys:  GClouder::Config::Arguments.required(%w(compute url-maps)),
              permitted_keys: GClouder::Config::Arguments.permitted(%w(compute url-maps)),
            )
          end

          def self.list
            Resources::Global.instances(path: %w(compute url_maps))
          end
        end

        module Remote
          def self.list
            Resources::Remote.instances(path: %w(compute url-maps))
          end
        end

        module URLMap
          include GClouder::GCloud

          def self.ensure(name, args = {}, silent: false)
            args = args.tap { |hs| hs.delete("path_matchers") }
            Resource.ensure :"compute url-maps", name, hash_to_args(args), silent: silent
          end
        end

        module PathMatcher
          include GClouder::GCloud

          def self.ensure(url_map_name, path_matcher)
            good "path_matcher: #{path_matcher["name"]}", indent: 4
            gcloud "compute url-maps add-path-matcher #{url_map_name} --path-matcher-name #{path_matcher["name"]} --default-backend-bucket #{path_matcher["default_backend_bucket"]} --new-hosts #{path_matcher["hosts"].join(",")}", silent: true
          end
        end
      end
    end
  end
end
