#!/usr/bin/env ruby

module GClouder
  module Resources
    module RuntimeConfig
      module Configs
        include GClouder::Config::CLIArgs
        include GClouder::Config::Project
        include GClouder::Logging
        include GClouder::Resource::Cleaner

        def self.header(stage = :ensure)
          info "[#{stage}] runtime-config / configs", indent: 1, title: true
        end

        def self.ensure
          return if Local.list.empty?
          header

          Local.list.each do |region, configs|
            info region, indent: 2, heading: true
            info
            configs.each do |config|
              ConfigObject.ensure(config["name"])
            end
          end
        end

        def self.validate
          return if Local.list.empty?
          header :validate
          Local.validate
        end

        module Local
          include GClouder::Config::CLIArgs
          include GClouder::Config::Project
          include GClouder::Logging

          # FIXME: improve validation
          def self.validate
            Resources::Validate::Region.instances(
              list,
              required_keys:  GClouder::Config::Arguments.required(%w(runtime-config configs)),
              permitted_keys: GClouder::Config::Arguments.permitted(%w(runtime-config configs))
            )
          end

          def self.list
            GClouder::Resources::Global.instances(path: %w(runtime-config configs))
          end
        end

        module Remote
          def self.list
            Resources::Remote.instances(
              path: %w(beta runtime-config configs)
            )
          end
        end

        module ConfigObject
          include GClouder::GCloud

          def self.ensure(config)
            Resource.ensure :"beta runtime-config configs", config
          end

          def self.purge(config)
            Resource.purge :"beta runtime-config configs", config
          end
        end
      end
    end
  end
end
