#!/usr/bin/env ruby

module GClouder
  module Resources
    module RuntimeConfig
      module Variables
        include GClouder::Config::CLIArgs
        include GClouder::Config::Project
        include GClouder::Logging
        include GClouder::Resource::Cleaner

        def self.header(stage = :ensure)
          info "[#{stage}] runtime-config / variables", indent: 1, title: true
        end

        def self.ensure
          return if Local.list.empty?
          header

          Local.list.each do |region, variables|
            info region, indent: 2, heading: true
            info
            variables.each do |variable|
              Variable.ensure(variable)
            end
          end

          puts "Ensure all done"
        end

        def self.validate
          return if Local.list.empty?
          header :validate
          Local.validate
        end

        module Local
          include GClouder::Config::CLIArgs
          include GClouder::Config::Project
          include GClouder::Logging

          def self.validate
            Resources::Validate::Region.instances(
              list,
              required_keys:  GClouder::Config::Arguments.required(%w(runtime-config configs variables)),
              permitted_keys: GClouder::Config::Arguments.permitted(%w(runtime-config configs variables))
            )
          end

          def self.list
            GClouder::Resources::Global.instances(path: %w(runtime-config variables))
          end
        end

        module Variable
          include GClouder::GCloud

          def self.args(variable)
            hash_to_args(variable.select{|k,v| k != "value"}) # Value is passed as an arg
          end

          def self.ensure(variable)
            Resource.set :"beta runtime-config configs variables", variable["name"], variable["value"], args(variable)
          end

          def self.purge(config)
            Resource.purge :"beta runtime-config configs variables", config
          end
        end
      end
    end
  end
end
